@@ -1,6 +1,6 @@
 /*
     scardcontrol.c: sample code to use/test SCardControl() API
-    Copyright (C) 2004-2007   Ludovic Rousseau
+    Copyright (C) 2004-2008   Ludovic Rousseau
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -12,13 +12,13 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+	You should have received a copy of the GNU General Public License along
+	with this program; if not, write to the Free Software Foundation, Inc., 51
+	Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 /*
- * $Id: scardcontrol.c 2653 2007-10-15 07:45:32Z rousseau $
+ * $Id: scardcontrol.c 2974 2008-05-28 18:32:52Z rousseau $
  */
 
 #include <stdio.h>
@@ -94,7 +94,7 @@ int main(int argc, char *argv[])
 #endif
 
 	printf("SCardControl sample code\n");
-	printf("V 1.2 2004-2007, Ludovic Rousseau <ludovic.rousseau@free.fr>\n");
+	printf("V 1.3 2004-2008, Ludovic Rousseau <ludovic.rousseau@free.fr>\n");
 
 	printf("\nTHIS PROGRAM IS NOT DESIGNED AS A TESTING TOOL!\n");
 	printf("Do NOT use it unless you really know what you do.\n\n");
@@ -199,7 +199,7 @@ int main(int argc, char *argv[])
 	rv = SCardStatus(hCard, pbReader, &dwReaderLen, &dwState, &dwProt,
 		pbAtr, &dwAtrLen);
 	printf(" Reader: %s (length %ld bytes)\n", pbReader, dwReaderLen);
-	printf(" State: 0x%lX\n", dwState);
+	printf(" State: 0x%04lX\n", dwState);
 	printf(" Prot: %ld\n", dwProt);
 	printf(" ATR (length %ld bytes):", dwAtrLen);
 	for (i=0; i<dwAtrLen; i++)
@@ -207,6 +207,12 @@ int main(int argc, char *argv[])
 	printf("\n");
 	PCSC_ERROR_CONT(rv, "SCardStatus")
 
+	if (dwState & SCARD_ABSENT)
+	{
+		printf("No card inserted\n");
+		goto end;
+	}
+
 	/* does the reader support PIN verification? */
 	rv = SCardControl(hCard, CM_IOCTL_GET_FEATURE_REQUEST, NULL, 0,
 		bRecvBuffer, sizeof(bRecvBuffer), &length);
