@@ -12,13 +12,13 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     Lesser General Public License for more details.
 
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
+	You should have received a copy of the GNU Lesser General Public License
+	along with this library; if not, write to the Free Software Foundation,
+	Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 /*
- * $Id: ccid.c 2596 2007-07-03 16:10:28Z rousseau $
+ * $Id: ccid.c 2974 2008-05-28 18:32:52Z rousseau $
  */
 
 #include <stdio.h>
@@ -33,6 +33,7 @@
 #include "defs.h"
 #include "ccid_ifdhandler.h"
 #include "commands.h"
+#include "ccid_usb.h"
 
 /*****************************************************************************
  *
@@ -278,6 +279,12 @@ int ccid_open_hack(unsigned int reader_index)
 			sleep(1);
 			ccid_descriptor->readTimeout = 60; /* 60 seconds */
 			break;
+
+		case SEG:
+#ifndef TWIN_SERIAL
+			InterruptRead(reader_index);
+#endif
+			break;
 	}
 
 	/* ICCD type A */
