@@ -12,13 +12,13 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     Lesser General Public License for more details.
 
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
+	You should have received a copy of the GNU Lesser General Public License
+	along with this library; if not, write to the Free Software Foundation,
+	Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 /*
- * $Id: ccid.h 2685 2007-11-15 23:10:36Z rousseau $
+ * $Id: ccid.h 2974 2008-05-28 18:32:52Z rousseau $
  */
 
 typedef struct
@@ -95,6 +95,19 @@ typedef struct
 	 * bInterfaceProtocol (CCID, ICCD-A, ICCD-B)
 	 */
 	int bInterfaceProtocol;
+
+	/*
+	 * bNumEndpoints
+	 */
+	int bNumEndpoints;
+
+	/*
+	 * GemCore SIM PRO slot status management
+	 * The reader always reports a card present even if no card is inserted.
+	 * If the Power Up fails the driver will report IFD_ICC_NOT_PRESENT instead
+	 * of IFD_ICC_PRESENT
+	 */
+	int dwSlotStatus;
 } _ccid_descriptor;
 
 /* Features from dwFeatures */
@@ -148,6 +161,8 @@ typedef struct
 #define CHERRYXX44	0x046a0010
 #define CL1356D		0x0B810200
 #define REINER_SCT	0x0C4B0300 
+#define SEG			0x08E68000
+#define BLUDRIVEII_CCID	0x1B0E1078
 
 /*
  * The O2Micro OZ776S reader has a wrong USB descriptor
