@@ -14,13 +14,13 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     Lesser General Public License for more details.
 
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
+	You should have received a copy of the GNU Lesser General Public License
+	along with this library; if not, write to the Free Software Foundation,
+	Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 /*
- * $Id: ccid_serial.c 2152 2006-09-06 20:06:58Z rousseau $
+ * $Id: ccid_serial.c 2983 2008-06-08 08:31:47Z rousseau $
  */
 
 #include <stdio.h>
@@ -172,7 +172,7 @@ static int get_bytes(unsigned int reader_index, unsigned char *buffer,
 status_t WriteSerial(unsigned int reader_index, unsigned int length,
 	unsigned char *buffer)
 {
-	int i;
+	unsigned int i;
 	unsigned char lrc;
 	unsigned char low_level_buffer[GEMPCTWIN_MAXBUF];
 
@@ -219,7 +219,7 @@ status_t WriteSerial(unsigned int reader_index, unsigned int length,
  *
  *****************************************************************************/
 status_t ReadSerial(unsigned int reader_index,
-	/*@unused@*/ unsigned int *length, unsigned char *buffer)
+	unsigned int *length, unsigned char *buffer)
 {
 	unsigned char c;
 	int rv;
@@ -333,6 +333,9 @@ ack:
 		goto start;
 	}
 
+	/* length of data read */
+	*length = to_read;
+
 	return STATUS_SUCCESS;
 } /* ReadSerial */
 
@@ -585,6 +588,7 @@ static status_t set_ccid_descriptor(unsigned int reader_index,
 	serialDevice[reader_index].ccid.dwMaxDataRate = 344086;
 	serialDevice[reader_index].ccid.bMaxSlotIndex = 0;
 	serialDevice[reader_index].ccid.arrayOfSupportedDataRates = SerialTwinDataRates;
+	serialDevice[reader_index].ccid.dwSlotStatus = IFD_ICC_PRESENT;
 	serialDevice[reader_index].echo = TRUE;
 
 	/* change some values depending on the reader */
