@@ -12,15 +12,18 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     Lesser General Public License for more details.
 
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
+	You should have received a copy of the GNU Lesser General Public License
+	along with this library; if not, write to the Free Software Foundation,
+	Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 /*
- * $Id: ccid_serial.h 1399 2005-03-14 18:56:53Z rousseau $
+ * $Id: ccid_serial.h 2974 2008-05-28 18:32:52Z rousseau $
  */
 
+#ifndef __CCID_SERAL_H__
+#define __CCID_SERAL_H__
+
 status_t OpenSerial(unsigned int reader_index, int channel);
 
 status_t OpenSerialByName(unsigned int reader_index, char *dev_name);
@@ -33,3 +36,4 @@ status_t ReadSerial(unsigned int reader_index, unsigned int *length,
 
 status_t CloseSerial(unsigned int reader_index);
 
+#endif
