@@ -1,6 +1,6 @@
 /*
     ccid_usb.c: USB access routines using the libusb library
-    Copyright (C) 2003-2004   Ludovic Rousseau
+    Copyright (C) 2003-2008   Ludovic Rousseau
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -12,13 +12,13 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     Lesser General Public License for more details.
 
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
+	You should have received a copy of the GNU Lesser General Public License
+	along with this library; if not, write to the Free Software Foundation,
+	Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 /*
- * $Id: ccid_usb.c 2691 2007-11-16 13:26:40Z rousseau $
+ * $Id: ccid_usb.c 3027 2008-06-26 13:42:22Z rousseau $
  */
 
 #define __CCID_USB__
@@ -30,6 +30,7 @@
 # include <sys/types.h>
 # endif
 #include <usb.h>
+#include <ifdhandler.h>
 
 #include "misc.h"
 #include "ccid.h"
@@ -67,6 +68,7 @@ typedef struct
 	 */
 	int bulk_in;
 	int bulk_out;
+	int interrupt;
 
 	/* Number of slots using the same device */
 	int real_nb_opened_slots;
@@ -115,7 +117,6 @@ static struct _bogus_firmware Bogus_firmwares[] = {
 
 	/* the firmware version is not correct since I do not have received a
 	 * working reader yet */
-	{ 0x0C4B, 0x0300, 0x0101 },	/* Reiner-SCT cyberJack pinpad(a) */
 #ifndef O2MICRO_OZ776_PATCH
 	{ 0x0b97, 0x7762, 0x0111 },	/* Oz776S */
 #endif
@@ -156,6 +157,7 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 
 	DEBUG_COMM3("Reader index: %X, Device: %s", reader_index, device);
 
+#ifndef __APPLE__
 	/* device name specified */
 	if (device)
 	{
@@ -200,6 +202,7 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 			}
 		}
 	}
+#endif
 
 	if (busses == NULL)
 		usb_init();
@@ -270,10 +273,16 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 		/* go to next supported reader for next round */
 		alias++;
 
+#ifndef __APPLE__
 		/* the device was specified but is not the one we are trying to find */
 		if (device
 			&& (vendorID != device_vendor || productID != device_product))
 			continue;
+#else
+		/* Leopard puts the friendlyname in the device argument */
+		if (device && strcmp(device, keyValue))
+			continue;
+#endif
 
 		/* on any USB buses */
 		for (bus = busses; bus; bus = bus->next)
@@ -332,6 +341,8 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 
 							*usbDevice[reader_index].nb_opened_slots += 1;
 							usbDevice[reader_index].ccid.bCurrentSlotIndex++;
+							usbDevice[reader_index].ccid.dwSlotStatus =
+								IFD_ICC_PRESENT;
 							DEBUG_INFO2("Opening slot: %d",
 								usbDevice[reader_index].ccid.bCurrentSlotIndex);
 							goto end;
@@ -433,6 +444,8 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 					usbDevice[reader_index].ccid.readTimeout = DEFAULT_COM_READ_TIMEOUT;
 					usbDevice[reader_index].ccid.arrayOfSupportedDataRates = get_data_rates(reader_index, dev);
 					usbDevice[reader_index].ccid.bInterfaceProtocol = usb_interface->altsetting->bInterfaceProtocol;
+					usbDevice[reader_index].ccid.bNumEndpoints = usb_interface->altsetting->bNumEndpoints;
+					usbDevice[reader_index].ccid.dwSlotStatus = IFD_ICC_PRESENT;
 					goto end;
 				}
 			}
@@ -496,6 +509,7 @@ status_t ReadUSB(unsigned int reader_index, unsigned int * length,
 	int rv;
 	char debug_header[] = "<- 121234 ";
 	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
+	int duplicate_frame = 0;
 
 read_again:
 	sprintf(debug_header, "<- %06X ", (int)reader_index);
@@ -525,6 +539,12 @@ read_again:
 	if ((*length >= BSEQ_OFFSET)
 		&& (buffer[BSEQ_OFFSET] < *ccid_descriptor->pbSeq -1))
 	{
+		duplicate_frame++;
+		if (duplicate_frame > 10)
+		{
+			DEBUG_CRITICAL("Too many duplicate frame detected");
+			return STATUS_UNSUCCESSFUL;
+		}
 		DEBUG_INFO("Duplicate frame detected");
 		goto read_again;
 	}
@@ -612,6 +632,13 @@ static int get_end_points(struct usb_device *dev, _usbDevice *usbdevice)
 	 */
 	for (i=0; i<usb_interface->altsetting->bNumEndpoints; i++)
 	{
+		/* interrupt end point (if available) */
+		if (usb_interface->altsetting->endpoint[i].bmAttributes == USB_ENDPOINT_TYPE_INTERRUPT)
+		{
+			usbdevice->interrupt = usb_interface->altsetting->endpoint[i].bEndpointAddress;
+			continue;
+		}
+
 		if (usb_interface->altsetting->endpoint[i].bmAttributes != USB_ENDPOINT_TYPE_BULK)
 			continue;
 
@@ -660,7 +687,7 @@ static int get_end_points(struct usb_device *dev, _usbDevice *usbdevice)
 	readerID = (dev->descriptor.idVendor << 16) + dev->descriptor.idProduct;
 	if (usb_interface != NULL
 		&& ((OZ776 == readerID) || (OZ776_7772 == readerID)
-		|| (REINER_SCT == readerID))
+		|| (REINER_SCT == readerID) || (BLUDRIVEII_CCID == readerID))
 		&& (0 == usb_interface->altsetting->extralen)) /* this is the bug */
 	{
 		for (i=0; i<usb_interface->altsetting->bNumEndpoints; i++)
@@ -692,7 +719,7 @@ static int get_end_points(struct usb_device *dev, _usbDevice *usbdevice)
  ****************************************************************************/
 int ccid_check_firmware(struct usb_device *dev)
 {
-	int i;
+	unsigned int i;
 
 	for (i=0; i<sizeof(Bogus_firmwares)/sizeof(Bogus_firmwares[0]); i++)
 	{
@@ -726,6 +753,7 @@ int ccid_check_firmware(struct usb_device *dev)
 	return FALSE;
 } /* ccid_check_firmware */
 
+
 /*****************************************************************************
  *
  *                                      get_data_rates
@@ -792,8 +820,14 @@ static unsigned int *get_data_rates(unsigned int reader_index,
 	int_array[i] = 0;
 
 	return int_array;
-}
+} /* get_data_rates */
 
+
+/*****************************************************************************
+ *
+ *                                      ControlUSB
+ *
+ ****************************************************************************/
 int ControlUSB(int reader_index, int requesttype, int request, int value,
 	unsigned char *bytes, unsigned int size)
 {
@@ -812,5 +846,46 @@ int ControlUSB(int reader_index, int requesttype, int request, int value,
 		DEBUG_XXD("receive: ", bytes, ret);
 
 	return ret;
-}
+} /* ControlUSB */
+
+/*****************************************************************************
+ *
+ *					InterruptRead
+ *
+ *
+ ****************************************************************************/
+int InterruptRead(int reader_index)
+{
+	int ret;
+	char buffer[8];
+	int timeout = 2*1000; /* 2 seconds */
+	static int hasfailed = FALSE;
+
+	if (hasfailed)
+	{
+		DEBUG_COMM("driver has failed");
+		return 0;
+	}
+
+	DEBUG_COMM("before");
+	ret = usb_interrupt_read(usbDevice[reader_index].handle,
+		usbDevice[reader_index].interrupt, buffer, sizeof(buffer), timeout);
+	DEBUG_COMM2("after %d", ret);
+
+	if (0 == ret)
+		hasfailed = TRUE;
+
+	if (ret < 0)
+	{
+		/* if usb_interrupt_read() times out we get EILSEQ or EAGAIN */
+		if ((errno != EILSEQ) && (errno != EAGAIN) && (errno != 0))
+			DEBUG_COMM4("usb_interrupt_read(%s/%s): %s",
+					usbDevice[reader_index].dirname,
+					usbDevice[reader_index].filename, strerror(errno));
+	}
+	else
+		DEBUG_XXD("NotifySlotChange: ", buffer, ret);
+
+	return ret;
+} /* InterruptRead */
 
