@@ -12,13 +12,13 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     Lesser General Public License for more details.
 
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
+	You should have received a copy of the GNU Lesser General Public License
+	along with this library; if not, write to the Free Software Foundation,
+	Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 /*
- * $Id: ccid_usb.h 2427 2007-02-23 16:49:50Z rousseau $
+ * $Id: ccid_usb.h 2974 2008-05-28 18:32:52Z rousseau $
  */
 
 #ifndef __CCID_USB_H__
@@ -42,4 +42,5 @@ status_t CloseUSB(unsigned int reader_index);
 int ControlUSB(int reader_index, int requesttype, int request, int value,
 	unsigned char *bytes, unsigned int size);
 
+int InterruptRead(int reader_index);
 #endif
