@@ -12,13 +12,13 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     Lesser General Public License for more details.
 
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
+	You should have received a copy of the GNU Lesser General Public License
+	along with this library; if not, write to the Free Software Foundation,
+	Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 /*
- * $Id: commands.c 2602 2007-08-02 20:04:12Z rousseau $
+ * $Id: commands.c 3004 2008-06-16 08:35:10Z rousseau $
  */
 
 #include <string.h>
@@ -28,6 +28,7 @@
 #include <ifdhandler.h>
 #include <reader.h>
 
+#include "misc.h"
 #include "commands.h"
 #include "openct/proto-t1.h"
 #include "ccid.h"
@@ -85,6 +86,7 @@ RESPONSECODE CmdPowerOn(unsigned int reader_index, unsigned int * nlength,
 	RESPONSECODE return_value = IFD_SUCCESS;
 	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
 
+#ifndef TWIN_SERIAL
 	if (ICCD_A == ccid_descriptor->bInterfaceProtocol)
 	{
 		int r;
@@ -162,6 +164,7 @@ RESPONSECODE CmdPowerOn(unsigned int reader_index, unsigned int * nlength,
 
 		return IFD_SUCCESS;
 	}
+#endif
 
 	/* store length of buffer[] */
 	length = *nlength;
@@ -738,6 +741,7 @@ RESPONSECODE CmdPowerOff(unsigned int reader_index)
 	RESPONSECODE return_value = IFD_SUCCESS;
 	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
 
+#ifndef TWIN_SERIAL
 	if (ICCD_A == ccid_descriptor->bInterfaceProtocol)
 	{
 		int r;
@@ -782,6 +786,7 @@ RESPONSECODE CmdPowerOff(unsigned int reader_index)
 
 		return IFD_SUCCESS;
 	}
+#endif
 
 	cmd[0] = 0x63; /* IccPowerOff */
 	cmd[1] = cmd[2] = cmd[3] = cmd[4] = 0;	/* dwLength */
@@ -827,6 +832,7 @@ RESPONSECODE CmdGetSlotStatus(unsigned int reader_index, unsigned char buffer[])
 	RESPONSECODE return_value = IFD_SUCCESS;
 	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
 
+#ifndef TWIN_SERIAL
 	if (ICCD_A == ccid_descriptor->bInterfaceProtocol)
 	{
 		int r;
@@ -900,6 +906,7 @@ again_status:
 		}
 		return IFD_SUCCESS;
 	}
+#endif
 
 	cmd[0] = 0x65; /* GetSlotStatus */
 	cmd[1] = cmd[2] = cmd[3] = cmd[4] = 0;	/* dwLength */
@@ -1015,6 +1022,7 @@ RESPONSECODE CCID_Transmit(unsigned int reader_index, unsigned int tx_length,
 	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
 	status_t ret;
 
+#ifndef TWIN_SERIAL
 	if (ICCD_A == ccid_descriptor->bInterfaceProtocol)
 	{
 		int r;
@@ -1054,6 +1062,7 @@ RESPONSECODE CCID_Transmit(unsigned int reader_index, unsigned int tx_length,
 
 		return IFD_SUCCESS;
 	}
+#endif
 
 	cmd[0] = 0x6F; /* XfrBlock */
 	i2dw(tx_length, cmd+1);	/* APDU length */
@@ -1093,9 +1102,11 @@ RESPONSECODE CCID_Receive(unsigned int reader_index, unsigned int *rx_length,
 	unsigned char cmd[10+CMD_BUF_SIZE];	/* CCID + APDU buffer */
 	unsigned int length;
 	RESPONSECODE return_value = IFD_SUCCESS;
-	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
 	status_t ret;
 
+#ifndef TWIN_SERIAL
+	_ccid_descriptor *ccid_descriptor = get_ccid_descriptor(reader_index);
+
 	if (ICCD_A == ccid_descriptor->bInterfaceProtocol)
 	{
 		int r;
@@ -1117,6 +1128,8 @@ RESPONSECODE CCID_Receive(unsigned int reader_index, unsigned int *rx_length,
 	{
 		int r;
 		unsigned char rx_tmp[4];
+		unsigned char *old_rx_buffer = NULL;
+		int old_rx_length = 0;
 
 		/* read a nul block. buffer need to be at least 4-bytes */
 		if (NULL == rx_buffer)
@@ -1125,6 +1138,15 @@ RESPONSECODE CCID_Receive(unsigned int reader_index, unsigned int *rx_length,
 			*rx_length = sizeof(rx_tmp);
 		}
 
+		/* the buffer must be 4 bytes minimum for ICCD-B */
+		if (*rx_length < 4)
+		{
+			old_rx_buffer = rx_buffer;
+			old_rx_length = *rx_length;
+			rx_buffer = rx_tmp;
+			*rx_length = sizeof(rx_tmp);
+		}
+
 time_request_ICCD_B:
 		/* Data Block */
 		r = ControlUSB(reader_index, 0xA1, 0x6F, 0, rx_buffer, *rx_length);
@@ -1136,6 +1158,13 @@ time_request_ICCD_B:
 			return IFD_COMMUNICATION_ERROR;
 		}
 
+		/* copy from the 4 bytes buffer if used */
+		if (old_rx_buffer)
+		{
+			memcpy(old_rx_buffer, rx_buffer, min(r, old_rx_length));
+			rx_buffer = old_rx_buffer;
+		}
+
 		/* bResponseType */
 		switch (rx_buffer[0])
 		{
@@ -1184,6 +1213,7 @@ time_request_ICCD_B:
 
 		return IFD_SUCCESS;
 	}
+#endif
 
 time_request:
 	length = sizeof(cmd);
@@ -1236,6 +1266,14 @@ time_request:
 		goto time_request;
 	}
 
+	/* we have read less (or more) data than the CCID frame says to contain */
+	if (length-10 != dw2i(cmd, 1))
+	{
+		DEBUG_CRITICAL3("Can't read all data (%d out of %d expected)",
+			length-10, dw2i(cmd, 1));
+		return_value = IFD_COMMUNICATION_ERROR;
+	}
+
 	length = dw2i(cmd, 1);
 	if (length <= *rx_length)
 		*rx_length = length;
@@ -1245,7 +1283,15 @@ time_request:
 		length = *rx_length;
 		return_value = IFD_ERROR_INSUFFICIENT_BUFFER;
 	}
-	memcpy(rx_buffer, cmd+10, length);
+
+	/* Kobil firmware bug. No support for chaining */
+	if (length && (NULL == rx_buffer))
+	{
+		DEBUG_CRITICAL2("Nul block expected but got %d bytes", length);
+		return_value = IFD_COMMUNICATION_ERROR;
+	}
+	else
+		memcpy(rx_buffer, cmd+10, length);
 
 	/* Extended case?
 	 * Only valid for RDR_to_PC_DataBlock frames */
