@@ -1,6 +1,6 @@
 /*
     ifdhandler.c: IFDH API
-    Copyright (C) 2003-2005   Ludovic Rousseau
+    Copyright (C) 2003-2008   Ludovic Rousseau
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -12,12 +12,12 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     Lesser General Public License for more details.
 
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
+	You should have received a copy of the GNU Lesser General Public License
+	along with this library; if not, write to the Free Software Foundation,
+	Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
-/* $Id: ifdhandler.c 2648 2007-10-11 13:41:35Z rousseau $ */
+/* $Id: ifdhandler.c 3025 2008-06-26 13:38:50Z rousseau $ */
 
 #include <stdio.h>
 #include <string.h>
@@ -57,6 +57,10 @@ int PowerOnVoltage = VOLTAGE_5V;
 static int DebugInitialized = FALSE;
 
 /* local functions */
+#if HAVE_DECL_TAG_IFD_POLLING_THREAD && !defined(TWIN_SERIAL) && defined(USE_USB_INTERRUPT)
+static RESPONSECODE IFDHPolling(DWORD Lun);
+static RESPONSECODE IFDHSleep(DWORD Lun);
+#endif
 static void init_driver(void);
 static void extra_egt(ATR_t *atr, _ccid_descriptor *ccid_desc, DWORD Protocol);
 static char find_baud_rate(unsigned int baudrate, unsigned int *list);
@@ -246,6 +250,38 @@ EXTERNAL RESPONSECODE IFDHCloseChannel(DWORD Lun)
 } /* IFDHCloseChannel */
 
 
+#if HAVE_DECL_TAG_IFD_POLLING_THREAD && !defined(TWIN_SERIAL) && defined(USE_USB_INTERRUPT)
+static RESPONSECODE IFDHPolling(DWORD Lun)
+{
+	int reader_index;
+	int ret;
+
+	if (-1 == (reader_index = LunToReaderIndex(Lun)))
+		return IFD_COMMUNICATION_ERROR;
+
+	/* log only if DEBUG_LEVEL_PERIODIC is set */
+	if (LogLevel & DEBUG_LEVEL_PERIODIC)
+		DEBUG_INFO2("lun: %X", Lun);
+
+	ret = InterruptRead(reader_index);
+	if (ret > 0)
+		return IFD_SUCCESS;
+	if (0 == ret)
+		return IFD_NO_SUCH_DEVICE;
+	return IFD_COMMUNICATION_ERROR;
+}
+
+/* on an ICCD device the card is always inserted
+ * so no card movement will ever happen: just do nothing */
+static RESPONSECODE IFDHSleep(DWORD Lun)
+{
+	DEBUG_INFO2("lun: %X", Lun);
+	sleep(60*60);	/* 1 hour */
+	return IFD_SUCCESS;
+}
+#endif
+
+
 EXTERNAL RESPONSECODE IFDHGetCapabilities(DWORD Lun, DWORD Tag,
 	PDWORD Length, PUCHAR Value)
 {
@@ -300,7 +336,11 @@ EXTERNAL RESPONSECODE IFDHGetCapabilities(DWORD Lun, DWORD Tag,
 			if (*Length >= 1)
 			{
 				*Length = 1;
+#ifdef __APPLE__
+				*Value = 0; /* Apple pcscd is bogus (rdar://problem/5697388) */
+#else
 				*Value = 1; /* Can talk to multiple readers at the same time */
+#endif
 			}
 			break;
 #endif
@@ -344,6 +384,36 @@ EXTERNAL RESPONSECODE IFDHGetCapabilities(DWORD Lun, DWORD Tag,
 				*(uint32_t *)Value = get_ccid_descriptor(reader_index) -> dwMaxCCIDMessageLength -10;
 			break;
 
+#if HAVE_DECL_TAG_IFD_POLLING_THREAD && !defined(TWIN_SERIAL) && defined(USE_USB_INTERRUPT)
+		case TAG_IFD_POLLING_THREAD:
+			{
+				_ccid_descriptor *ccid_desc;
+
+				/* default value: not supported */
+				*Length = 0;
+
+				ccid_desc = get_ccid_descriptor(reader_index);
+				/* CCID and not ICCD */
+				if ((0 == ccid_desc -> bInterfaceProtocol)
+					/* 3 end points */
+					&& (3 == ccid_desc -> bNumEndpoints))
+				{
+					*Length = sizeof(void *);
+					if (Value)
+						*(void **)Value = IFDHPolling;
+				}
+
+				if ((ICCD_A == ccid_desc->bInterfaceProtocol)
+					|| (ICCD_B == ccid_desc->bInterfaceProtocol))
+				{
+					*Length = sizeof(void *);
+					if (Value)
+						*(void **)Value = IFDHSleep;
+				}
+			}
+			break;
+#endif
+
 		default:
 			return IFD_ERROR_TAG;
 	}
@@ -431,7 +501,7 @@ EXTERNAL RESPONSECODE IFDHSetProtocolParameters(DWORD Lun, DWORD Protocol,
 	/* Do not send CCID command SetParameters or PPS to the CCID
 	 * The CCID will do this himself */
 	if (ccid_desc->dwFeatures & CCID_CLASS_AUTO_PPS_PROP)
-		return IFD_SUCCESS;
+		goto end;
 
 	/* Get ATR of the card */
 	ATR_InitFromArray(&atr, ccid_slot->pcATRBuffer, ccid_slot->nATRLength);
@@ -626,12 +696,24 @@ EXTERNAL RESPONSECODE IFDHSetProtocolParameters(DWORD Lun, DWORD Protocol,
 		/* if the requested protocol is not the default one
 		 * or a TA1/PPS1 is present */
 		if (((pps[1] & 0x0F) != default_protocol) || (PPS_HAS_PPS1(pps)))
+		{
+#ifdef O2MICRO_OZ776_PATCH
+			if ((OZ776 == ccid_desc->readerID)
+				|| (OZ776_7772 == ccid_desc->readerID))
+			{
+				Protocol = default_protocol;
+				DEBUG_INFO2("PPS not supported on O2Micro readers. Using T=%d",
+					Protocol);
+			}
+			else
+#endif
 			if (PPS_Exchange(reader_index, pps, &len, &pps[2]) != PPS_OK)
 			{
 				DEBUG_INFO("PPS_Exchange Failed");
 
 				return IFD_ERROR_PTS_FAILURE;
 			}
+		}
 	}
 
 	/* Now we must set the reader parameters */
@@ -786,6 +868,7 @@ EXTERNAL RESPONSECODE IFDHSetProtocolParameters(DWORD Lun, DWORD Protocol,
 		DEBUG_COMM3("T=1: IFSC=%d, IFSD=%d", t1->ifsc, t1->ifsd);
 	}
 
+end:
 	/* store used protocol for use by the secure commands (verify/change PIN) */
 	ccid_desc->cardProtocol = Protocol;
 
@@ -874,6 +957,10 @@ EXTERNAL RESPONSECODE IFDHPowerICC(DWORD Lun, DWORD Action,
 			if (CmdPowerOn(reader_index, &nlength, pcbuffer, PowerOnVoltage)
 				!= IFD_SUCCESS)
 			{
+				/* used by GemCore SIM PRO: no card is present */
+				get_ccid_descriptor(reader_index)->dwSlotStatus
+					= IFD_ICC_NOT_PRESENT;;
+
 				DEBUG_CRITICAL("PowerUp failed");
 				return_value = IFD_ERROR_POWER_ACTION;
 				goto end;
@@ -1103,6 +1190,12 @@ EXTERNAL RESPONSECODE IFDHICCPresence(DWORD Lun)
 
 	ccid_descriptor = get_ccid_descriptor(reader_index);
 
+	if (GEMCORESIMPRO == ccid_descriptor->readerID)
+	{
+		return_value = ccid_descriptor->dwSlotStatus;
+		goto end;
+	}
+
 	/* save the current read timeout computed from card capabilities */
 	oldReadTimeout = ccid_descriptor->readTimeout;
 
@@ -1207,6 +1300,7 @@ EXTERNAL RESPONSECODE IFDHICCPresence(DWORD Lun)
 		}
 	}
 
+end:
 	DEBUG_PERIODIC2("Card %s",
 		IFD_ICC_PRESENT == return_value ? "present" : "absent");
 
@@ -1224,6 +1318,9 @@ void init_driver(void)
 {
 	char keyValue[TOKEN_MAX_VALUE_SIZE];
 	char infofile[FILENAME_MAX];
+	char *e;
+
+	DEBUG_INFO("Driver version: " VERSION);
 
 	/* Info.plist full patch filename */
 	snprintf(infofile, sizeof(infofile), "%s/%s/Contents/Info.plist",
@@ -1239,6 +1336,16 @@ void init_driver(void)
 		DEBUG_INFO2("LogLevel: 0x%.4X", LogLevel);
 	}
 
+	e = getenv("LIBCCID_ifdLogLevel");
+	if (e)
+	{
+		/* convert from hex or dec or octal */
+		LogLevel = strtoul(e, NULL, 0);
+
+		/* print the log level used */
+		DEBUG_INFO2("LogLevel from LIBCCID_ifdLogLevel: 0x%.4X", LogLevel);
+	}
+	
 	/* Driver options */
 	if (0 == LTPBundleFindValueWithKey(infofile, "ifdDriverOptions", keyValue, 0))
 	{
