@@ -12,13 +12,13 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     Lesser General Public License for more details.
 
-    You should have received a copy of the GNU Lesser General Public
-    License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
+	You should have received a copy of the GNU Lesser General Public License
+	along with this library; if not, write to the Free Software Foundation,
+	Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
 /*
- * $Id: utils.c 2121 2006-08-03 13:12:20Z rousseau $
+ * $Id: utils.c 2974 2008-05-28 18:32:52Z rousseau $
  */
 
 #include <pcsclite.h>
@@ -39,10 +39,21 @@ void InitReaderIndex(void)
 		ReaderIndex[i] = -1;
 } /* InitReaderIndex */
 
-int GetNewReaderIndex(const DWORD Lun)
+int GetNewReaderIndex(const int Lun)
 {
 	int i;
 
+	/* check that Lun is NOT already used */
+	for (i=0; i<CCID_DRIVER_MAX_READERS; i++)
+		if (Lun == ReaderIndex[i])
+			break;
+
+	if (i < CCID_DRIVER_MAX_READERS)
+	{
+		DEBUG_CRITICAL2("Lun: %d is already used", Lun);
+		return -1;
+	}
+
 	for (i=0; i<CCID_DRIVER_MAX_READERS; i++)
 		if (-1 == ReaderIndex[i])
 		{
@@ -54,7 +65,7 @@ int GetNewReaderIndex(const DWORD Lun)
 	return -1;
 } /* GetReaderIndex */
 
-int LunToReaderIndex(const DWORD Lun)
+int LunToReaderIndex(const int Lun)
 {
 	int i;
 
